/*_############################################################################
  _## 
  _##  SNMP4J-LOG4J - Log4jLogFactory.java  
  _## 
  _##  Copyright (C) 2003-2020  Frank Fock (SNMP4J.org)
  _##  
  _##  Licensed under the Apache License, Version 2.0 (the "License");
  _##  you may not use this file except in compliance with the License.
  _##  You may obtain a copy of the License at
  _##  
  _##      http://www.apache.org/licenses/LICENSE-2.0
  _##  
  _##  Unless required by applicable law or agreed to in writing, software
  _##  distributed under the License is distributed on an "AS IS" BASIS,
  _##  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  _##  See the License for the specific language governing permissions and
  _##  limitations under the License.
  _##  
  _##########################################################################*/
package org.snmp4j.log;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.core.LoggerContext;

import java.util.Collection;
import java.util.Iterator;
import java.util.Collections;
import java.util.ArrayList;

/**
 * The {@code Log4jLogFactory} implements a SNMP4J LogFactory for
 * Log4J. In order to use Log4J for logging SNMP4J log messages the
 * static {@link LogFactory#setLogFactory} method has to be used before
 * any SNMP4J class is referenced or instantiated.
 *
 * @author Frank Fock
 * @version 2.8.2
 * @since 1.2.1
 */
public class Log4jLogFactory extends LogFactory {

    /**
     * Creates a Log4J log factory.
     */
    public Log4jLogFactory() {
    }

    protected LogAdapter createLogger(Class c) {
        return new Log4jLogAdapter(LogManager.getLogger(c));
    }

    protected LogAdapter createLogger(String className) {
        return new Log4jLogAdapter(LogManager.getLogger(className));
    }

    public LogAdapter getRootLogger() {
        return new Log4jLogAdapter(LogManager.getRootLogger());
    }

    @SuppressWarnings("unchecked")
    public Iterator loggers() {
        Collection<org.apache.logging.log4j.core.Logger> l = LoggerContext.getContext(false).getLoggers();
        ArrayList<Log4jLogAdapter> la = new ArrayList<Log4jLogAdapter>(l.size());
        for (Logger logger : l) {
            la.add(new Log4jLogAdapter(logger));
        }
        Collections.sort(la);
        return la.iterator();
    }
}
